//-------------------------------------------------------------------
//!OtBbNNXwb_
//!@memo 08/05/29 mmm
//-------------------------------------------------------------------

#ifndef __MMM_GRAPHIC_H__
#define __MMM_GRAPHIC_H__

#include "MMMStd.h"

namespace mmm
{
	//DirectXIuWFNg̉p}N
	#define SafeReleaseD3D(p) if((p) != 0){ (p)->Release(); (p) = 0; }

	//DirectXIuWFNgfoCX̃|C^擾ׂ̃}N
	template<class T> LPDIRECT3DDEVICE9 GetDevicePtr(T pInterface)
	{
		LPDIRECT3DDEVICE9 p;
		pInterface->GetDevice(&p);
		return p;
	}

	//----------------------------------------------------------
	//!OtBbNNX
	//----------------------------------------------------------
	class Graphic
	{
	public:
		Graphic() : pD3D_(0), pDevice_(0){}
		virtual ~Graphic(){ MMM_DEBUG_CHECK(pD3D_ == 0); MMM_DEBUG_CHECK(pDevice_ == 0); }
	public:
		//FCC֐
		inline static unsigned int ColorRGB(unsigned char r, unsigned char g, unsigned char b)
		{
			return (0xff << 24) | (r << 16) | (g << 8) | (b);
		}
		inline static unsigned int ColorRGBA(unsigned char r, unsigned char g, unsigned char b, unsigned char a)
		{
			return (a << 24) | (r << 16) | (g << 8) | (b);
		}
	public:
		//!NA̓tO
		enum
		{
			CLEAR_IMAGE = 0x01,	//!<ʃNA
			CLEAR_Z = 0x02,		//!<yobt@NA
		};
	public:
		//!֐
		bool Create(HWND hWnd, int screenWidth, int screenHeight)
		{
			pD3D_ = Direct3DCreate9(D3D_SDK_VERSION);
			if(pD3D_ == 0) return false;

			//fBXvC[h̎擾
			D3DDISPLAYMODE dm ={0};
			if(FAILED(pD3D_->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &dm)))
			{
				SafeReleaseD3D(pD3D_);
				return false;
			}

			//foCX̐pp[^̏
			D3DPRESENT_PARAMETERS pp; 
			ZeroMemory(&pp, sizeof(pp));
			pp.Windowed = TRUE;	
			pp.SwapEffect = D3DSWAPEFFECT_DISCARD;
			pp.BackBufferFormat = dm.Format;
			pp.BackBufferWidth = screenWidth;
			pp.BackBufferHeight = screenHeight;
			pp.BackBufferCount = 1;
			pp.EnableAutoDepthStencil = TRUE;
			pp.AutoDepthStencilFormat = D3DFMT_D16;

			//foCX̐
			//̂玎
			if(FAILED(pD3D_->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL,
				hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, &pp, &pDevice_)))
			{
				if(FAILED(pD3D_->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_REF,
					hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, &pp, &pDevice_)))
				{
					if(FAILED(pD3D_->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_SW,
						hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, &pp, &pDevice_)))
					{
						SafeReleaseD3D(pD3D_);
						return false;
					}
				}
			}

			return true;
		}
		//!֐
		bool Release()
		{
			SafeReleaseD3D(pD3D_);
			SafeReleaseD3D(pDevice_);

			return true;
		}
	public:
		bool IsCreated(){ return pDevice_ != 0; }
	public:
		//!p̎擾֐
		LPDIRECT3DDEVICE9 GetDevice()
		{
			return pDevice_;
		}
	public:
		//!NA֐
		void Clear(int clearFlag, unsigned int color, float z)
		{
			MMM_DEBUG_CHECK(pDevice_);

			DWORD flag = 0;
			if(clearFlag & CLEAR_IMAGE) flag |= D3DCLEAR_TARGET;
			if(clearFlag & CLEAR_Z) flag |= D3DCLEAR_ZBUFFER;
			if(flag == 0) return;

			pDevice_->Clear(0, 0, flag, color, z, 0);
		}
	public:
		bool BeginScene()
		{
			MMM_DEBUG_CHECK(pDevice_);

			return SUCCEEDED(pDevice_->BeginScene());
		}
		bool EndScene()
		{
			MMM_DEBUG_CHECK(pDevice_);
			return SUCCEEDED(pDevice_->EndScene());
		}
		void Present()
		{
			MMM_DEBUG_CHECK(pDevice_);

			pDevice_->Present(0, 0, 0, 0);
		}
	private:
		LPDIRECT3D9 pD3D_;				//!<cRcIuWFNg
		LPDIRECT3DDEVICE9 pDevice_;		//!<foCX|C^
	};
	//----------------------------------------------------------
	//!fobOtHgNX
	//----------------------------------------------------------
	class DebugFont
	{
	public:
		//!eLXg̑̎
		enum
		{
			ALIGN_LEFT = 0x00,
			ALIGN_RIGHT = 0x01,
		};
	public:
		DebugFont() : pFont_(0){}
		virtual ~DebugFont(){ MMM_DEBUG_CHECK(pFont_ == 0); }
	public:
		//!֐
		bool Create(Graphic* pGrp, LONG width, LONG height, LONG weight, const char* faceName)
		{
			Release();

			D3DXFONT_DESC fd;
			fd.Width = width;
			fd.Height = height;
			fd.Weight = weight;
			fd.MipLevels = D3DX_DEFAULT;
			fd.Italic = FALSE;
			fd.CharSet = DEFAULT_CHARSET;
			fd.OutputPrecision = OUT_DEFAULT_PRECIS;
			fd.Quality = DEFAULT_QUALITY;
			fd.PitchAndFamily = DEFAULT_PITCH;
			strcpy_s(fd.FaceName, faceName);

			if(FAILED(D3DXCreateFontIndirect(pGrp->GetDevice(), &fd, &pFont_)))
			{
				return false;
			}

			return true;
		}
		//!֐
		bool Release()
		{
			SafeReleaseD3D(pFont_);
			return true;
		}
		//!`֐
		void DrawText(float x, float y, int dtFormat, DWORD color, const char* format, ...)
		{
			MMM_DEBUG_CHECK(pFont_ != 0);

			char va_str[512];
			va_list list;
			va_start(list, format);
			vsprintf_s(va_str, format, list);

			LPDIRECT3DDEVICE9 pDev = GetDevicePtr(pFont_);
			if(pDev == 0) return;

			D3DVIEWPORT9 vp;
			pDev->GetViewport(&vp);

			RECT rect;
			SetRect(&rect, (int)x, (int)y, vp.Width, vp.Height);

			int flag = 0;
			if(dtFormat & ALIGN_RIGHT) flag |= DT_RIGHT;
			pFont_->DrawText(0, va_str, -1, &rect, flag, color);

			pDev->Release();
		}
	private:
		LPD3DXFONT pFont_;	//!<tHgIuWFNgւ̃|C^
	};
}

#endif